#include <stdio.h>
#include <string.h>
#include "NUC1xx.h"
#include "DrvSYS.h"
#include "DrvGPIO.h"
#include "DrvSPI.h"
#include "hw_config.h"
#include "enc28j60.h"
#include "ip_arp_udp_tcp.h"
#include "net.h"
/*Definisi MAC Address, IP, URL untuk GET, dan port HTTP*/

static uint8_t mymac[6] = {0x54,0x55,0x58,0x10,0x00,0x24};
static uint8_t myip[4] = {192,168,1,15};
static uint16_t mywwwport =80;

/*Definisi buffer untuk data*/
#define BUFFER_SIZE 400
static uint8_t buf[BUFFER_SIZE+1];

// Prototype Function
uint16_t print_webpage(uint8_t *buf);
void respond(void);

/*Kumpulan fungsi pada file main.c*/
uint16_t print_webpage(uint8_t *buf)//Menampilkan halaman WEB
{
        uint16_t plen;

		/*Content web OK tanpa fitur "no-cache"
        plen=fill_tcp_data_p(buf,0,PSTR("HTTP/1.0 200 OK\r\nContent-Type: text/html\r\n\r\n"));
		plen=fill_tcp_data_p(buf,plen,PSTR("<title>Web Server Demo</title>"));
        plen=fill_tcp_data_p(buf,plen,PSTR("<center><p><h1>Simple Web Server</h1></p> "));
		plen=fill_tcp_data_p(buf,plen,PSTR("<p>Selamat Datang di Web Server sederhana berbasis NUC120</p>"));
        plen=fill_tcp_data_p(buf,plen,PSTR("<p>Link <a href=\"http://www.InnovativeElectronics.com\">www.InnovativeElectronics.com</a></p>"));
		*/

		/*Content web dengan fitur "no-cache"*/
		plen=fill_tcp_data_p(buf,0,"HTTP/1.0 200 OK\r\nContent-Type: text/html\r\n\r\n");
		plen=fill_tcp_data_p(buf,plen,"<html><head><meta http-equiv=\"Pragma\" Content=\"no cache\">");
		plen=fill_tcp_data_p(buf,plen,"<title>Web Server Demo</title></head>");
		plen=fill_tcp_data_p(buf,plen,"<body><center><h1>Simple Web Server</h1><br>");
		plen=fill_tcp_data_p(buf,plen,"Selamat Datang di Web Server sederhana berbasis NUC120 dengan menggunakan EMS Ethernet Module & DT-ARM NUC120 Board <br>");
        plen=fill_tcp_data_p(buf,plen,"<p><a href=\"http://www.InnovativeElectronics.com\">IE.COM</a></p></body></html>");
        return(plen);
}

void setup(void)//Inisialisasi Sistem
{

   /*Inisialisasi ENC28j60*/
	enc28j60Init(mymac);

	enc28j60clkout(2); // change clkout from 6.25MHz to 12.5MHz

	delay_ms(10);

	/* Konfigurasi LED pada MAG JACK, (datasheet ENC28j60) */
	// LEDA=hijau LEDB=kuning
	// 0x880 is PHLCON LEDB=on, LEDA=on
	// enc28j60PhyWrite(PHLCON,0b0000 1000 1000 00 00);
	enc28j60PhyWrite(PHLCON,0x880);
	delay_ms(500);

	// 0x990 is PHLCON LEDB=off, LEDA=off
	// enc28j60PhyWrite(PHLCON,0b0000 1001 1001 00 00);
	enc28j60PhyWrite(PHLCON,0x990);
	delay_ms(500);

	// 0x880 is PHLCON LEDB=on, LEDA=on
	// enc28j60PhyWrite(PHLCON,0b0000 1000 1000 00 00);
	enc28j60PhyWrite(PHLCON,0x880);
	delay_ms(500);

	// 0x990 is PHLCON LEDB=off, LEDA=off
	// enc28j60PhyWrite(PHLCON,0b0000 1001 1001 00 00);
	enc28j60PhyWrite(PHLCON,0x990);
	delay_ms(500);

	// 0x476 is PHLCON LEDA=links status, LEDB=receive/transmit
	// enc28j60PhyWrite(PHLCON,0b0000 0100 0111 01 10);
	enc28j60PhyWrite(PHLCON,0x476);
	delay_ms(100);

	//Inisialisasi ethernet/ip layer:
	init_ip_arp_udp_tcp(mymac,myip,80);

}



int main(void)//Program Utama
{
	Set_System();

	//Deklarasi & Inisialisasi variabel yang digunakan
	uint16_t plen, dat_p;

	//inisialisasi Sistem
	setup();

    while(1)
	{
		plen = enc28j60PacketReceive(BUFFER_SIZE, buf);

		///Nilai plen !=0 jika ada paket yang valid (tanpa CRC error)
		if(plen!=0)
		{

			// arp is broadcast if unknown but a host may also verify the mac address by sending it to a unicast address.
			if(eth_type_is_arp_and_my_ip(buf,plen))
			{

				make_arp_answer_from_request(buf);
			}

			else if(buf[IP_PROTO_P]==IP_PROTO_ICMP_V && buf[ICMP_TYPE_P]==ICMP_TYPE_ECHOREQUEST_V)
			{

				make_echo_reply_from_request(buf,plen);
			}

			// tcp port www start, compare only the lower byte
			else if (buf[IP_PROTO_P]==IP_PROTO_TCP_V&&buf[TCP_DST_PORT_H_P]==0&&buf[TCP_DST_PORT_L_P]==mywwwport)
			{

				if (buf[TCP_FLAGS_P] & TCP_FLAGS_SYN_V)
				{

					make_tcp_synack_from_syn(buf); // make_tcp_synack_from_syn does already send the syn,ack
				}

				else if (buf[TCP_FLAGS_P] & TCP_FLAGS_ACK_V)
				{

					init_len_info(buf); // Inisialisasi beberapa struktur data
					dat_p=get_tcp_data_pointer();

					if (dat_p==0)//Kemungkinan hanya data ACK saja
					{

						if (buf[TCP_FLAGS_P] & TCP_FLAGS_FIN_V)
						{

							make_tcp_ack_from_any(buf);
						}
					}

					else if (strncmp("GET ",(char *)&(buf[dat_p]),4)!=0)
					{

						// head, post and other methods for possible status codes see:
						// http://www.w3.org/Protocols/rfc2616/rfc2616-sec10.html
						plen=fill_tcp_data_p(buf,0,"HTTP/1.0 200 OK\r\nContent-Type: text/html\r\n\r\n<h1>200 OK</h1>");
						make_tcp_ack_from_any(buf); // send ack for http get
						make_tcp_ack_with_data(buf,plen); // send data
					}

					else if (strncmp("GET ",(char *)&(buf[dat_p]),4)==0 && strncmp("/ ",(char *)&(buf[dat_p+4]),2)==0)//Jika ada request dengan metode "GET"
					{


						plen=print_webpage(buf);
						make_tcp_ack_from_any(buf); // send ack for http get
						make_tcp_ack_with_data(buf,plen); // send data
					}
				}
			}
		}


	}


}
